# -*- coding: utf-8 -*-
"""
tabledataextractor.input

Methods for input and interfacing with html,xml,cls,pdf readers.
The output of these methods is always a Numpy array.

jm2111@cam.ac.uk
~~~~~~~~~~~~~~~~~
"""







